import pygame, os
from pygame import *

#class for loading images
def load_image(name, colorkey=None, alpha=False):
    """loads an image into memory"""
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    if alpha == False:
        image = image.convert()
    else:
        image = image.convert_alpha()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()
    

#new mouse function
def mouse_down(button):
    for event in pygame.event.get(MOUSEBUTTONDOWN):
        if event.type == MOUSEBUTTONDOWN:
            if event.button == button:
                pygame.event.pump()
                return True
            else:
                pygame.event.pump()
                return False

            
#class for buttons!
class Button:
    def __init__(self, locx, locy, image_name=None, alpha = False):
        if image_name is not None:
            if alpha == False:
                self.image, self.rect = load_image(image_name, -1)
                self.reset_nesc = False
                self.highlighted_image = self.image.copy()
                self.highlighted_image = pygame.transform.scale(self.highlighted_image,(self.rect[2]*0.9,
                                                                                        self.rect[3]*0.95))
                self.highlighted_rect = self.highlighted_image.get_rect()
                self.is_image_i = True
            else:
                self.image, self.rect = load_image(image_name, alpha=True)
                self.reset_nesc = False
                self.highlighted_image = self.image.copy()
                self.highlighted_image = pygame.transform.scale(self.highlighted_image,(self.rect[2]*0.9,
                                                                                        self.rect[3]*0.95))
                self.highlighted_rect = self.highlighted_image.get_rect()
                self.is_image_i = True
        else:
            self.image = pygame.Surface((1, 1))
            self.rect = self.image.get_rect()
            self.highlighted_image = self.image.copy()
            self.highlighted_image = pygame.transform.scale(self.highlighted_image,(self.rect[2]*0.9,
                                                                                    self.rect[3]*0.95))
            self.highlighted_rect = self.highlighted_image.get_rect()
            self.reset_nesc = True
            self.is_image_i = False
            
        self.rect.centerx = locx
        self.rect.centery = locy
        self.x = locx
        self.y = locy
        
        self.color = 64,255,248
        self.highlight_color = self.color
        self.set_highlight_color()
        self.do_hl = False
        self.t_high_l = True
        self.font_size = 30
        self.font = pygame.font.Font(None,30)
        self.font_small = pygame.font.Font(None,int(self.font_size*0.75))
        self.message_surf = pygame.Surface((5,5))

    def toggle_highlighting(self, t):
        """on(T) or off[F]"""
        if t =='T':
            self.t_high_l = True
        else:
            self.t_high_l = False

    def clicked(self, m):
        self.mouse_point = pygame.Rect((pygame.mouse.get_pos()),(1,1))
        if m == True:
            if self.mouse_point.colliderect(self.rect):
                return True
            else:
                return False
        else:
            return False

    def set_highlight_color(self):
        a, b, c = self.color
        self.highlight_color = [0,0,0]

        self.highlight_color[0] = 255 - a
        self.highlight_color[1] = 255 - b
        self.highlight_color[2] = 255 - c

    def highlight(self):
        self.mouse_point = pygame.Rect((pygame.mouse.get_pos()),(1,1))
        if self.t_high_l == True:
            if self.mouse_point.colliderect(self.rect):
                self.do_hl = True
            else:
                self.do_hl = False
        else:
            self.do_hl = False

    def Set_Font(self,font=None,color=None):
        if font is not None:
            self.font = font
        self.font_size = self.font.size("a")[1]
        if color is not None:
            self.color = color
            self.set_highlight_color()

    def draw(self, screen, message):
        self.highlight()
        if self.is_image_i == True:
            if self.do_hl == False:
                screen.blit(self.image, self.rect)
                a = self.font.render(message,1,self.color)
                self.message_surf = a
            else:
                screen.blit(self.highlighted_image, (self.rect[0]+((self.rect[2] - self.highlighted_rect[2])/2),
                                                     self.rect[1]+((self.rect[3] - self.highlighted_rect[3])/2),
                                                     self.rect[2],
                                                     self.rect[3]))
                a = self.font.render(message,1,self.highlight_color)
                self.message_surf = a
            if self.reset_nesc == True:
                self.image = pygame.Surface((a.get_width(), a.get_height()))
                self.rect = a.get_rect()
                self.rect.centerx = self.x
                self.rect.centery = self.y
            cx = a.get_width()/2
            cy = a.get_height()/2
            if self.do_hl == False:
                screen.blit(a,(self.rect.centerx - cx,self.rect.centery - cy))
            else:
                screen.blit(a,(self.rect.centerx - cx,self.rect.centery - cy+((self.rect[3] - self.highlighted_rect[3])/2)))
            return

        else:
            if self.do_hl == False:
                screen.blit(self.image, self.rect)
                a = self.font.render(message,1,self.color)
                self.message_surf = a
            else:
                screen.blit(self.highlighted_image, self.rect)
                a = self.font.render(message,1,self.highlight_color)
                self.message_surf = a
            if self.reset_nesc == True:
                self.image = pygame.Surface((a.get_width(), a.get_height()))
                self.rect = a.get_rect()
                self.rect.centerx = self.x
                self.rect.centery = self.y
            cx = a.get_width()/2
            cy = a.get_height()/2
            screen.blit(a,(self.rect.centerx - cx,self.rect.centery - cy))
            return



#clas for a message box!
class MessageBox:
    def __init__(self, locx, locy, box_width, box_height, rows_per_page, max_horz_scroll=40, image_name=None):
        if image_name is not None:
            self.image, self.rect = load_image(image_name, -1)
        else:
            self.image = pygame.Surface((box_width, box_height))
            self.rect = self.image.get_rect()
        self.rect.centerx = locx
        self.rect.centery = locy
        self.messages = ''
        self.box_height = box_height
        self.box_width = box_width
        self.font_size = 20
        self.font = pygame.font.Font(None, 20)
        self.color = [255, 0 , 0]
        self.comp_messages = []
        self.mess_rows = 0
        self.max_text_rows_per_page = rows_per_page

        
        self.mess_num = 0
        self.scroll_vert_offset = 0
        self.scroll_horz_offset = 0
        self.on_message_num = 0
        self.max_h_scroll = max_horz_scroll

    
    def Set_Font(self,font,color):
        self.font = font
        self.font_size = self.font.size("a")[1]
        self.color = color

    def add_message(self, new_message):
        if self.mess_num >= self.max_text_rows_per_page:
            self.on_message_num += 1
        if not self.messages == '':
            self.messages = self.messages + new_message + '\n'
        else:
            self.messages = self.messages + '\n' + new_message + '\n'
        self.prep_messages()
        self.mess_num += 1

    def prep_messages(self):
        str_list = [""]
        str_n = 0
        for i in range(len(self.messages)):
            if not self.messages[i] == "\n":
                str_list[str_n] = str_list[str_n] + self.messages[i]
            else:
                str_list.append("")
                str_n+=1

        self.comp_messages = str_list
        self.mess_rows = str_n

    def scroll_vert(self, d):
        """Up(U) or Down(D)"""
        maxx_scroll = self.mess_rows + self.max_text_rows_per_page
        if d == 'U':
            if not self.on_message_num == -1:
                self.scroll_vert_offset += 1
                self.on_message_num -= 1
            else:
                pass
        else:
            if not self.on_message_num == self.mess_num - self.max_text_rows_per_page:
                self.scroll_vert_offset -= 1
                self.on_message_num += 1
            else:
                pass

    def scroll_horz(self, d):
        """Left(L) or Right(R)"""
        if d == "L":
            if not self.scroll_horz_offset <= 0:
                self.scroll_horz_offset -= 5

        else:
            if self.scroll_horz_offset < self.max_h_scroll:
                self.scroll_horz_offset += 5


    def draw(self, screen):
        self.message_surf = pygame.Surface((self.box_width, self.box_height))
        num = -1
        if self.mess_rows < self.max_text_rows_per_page:
            for i in range(self.mess_rows):
                b = self.font.render(self.comp_messages[i], 1, self.color)
                self.message_surf.blit(b, (0 - self.scroll_horz_offset, num * self.font_size))
                num += 1
        else:
            for i in range(self.mess_rows - self.max_text_rows_per_page - self.scroll_vert_offset, self.mess_rows):
                b = self.font.render(self.comp_messages[i], 1, self.color)
                self.message_surf.blit(b, (0 - self.scroll_horz_offset, num * self.font_size))
                num += 1

        self.image.blit(self.message_surf, (25, 25))
        screen.blit(self.image, (self.rect.centerx, self.rect.centery))
        return


#class for an input box!
class InputBox:
    def __init__(self, locx, locy, prompt, box_width, box_height, max_letters=40, box_image=None, prompt_image=None):
        self.prompt = prompt
        self.box_width = box_width
        self.box_height = box_height
        self.max_letters = max_letters
        if box_image is not None:
            self.box_image, self.b_image_rect = load_image(box_image, -1)
        else:
            self.box_image = pygame.Surface((box_width, box_height))
            self.b_image_rect = self.box_image.get_rect()

        if prompt_image is not None:
            self.prompt_image, self.p_image_rect = load_image(prompt_image, -1)
            self.p_image_rect.centerx = locx
            self.p_image_rect.centery = locy - self.prompt_image.get_height() / 2
            self.reset_nesc = False
        else:
            self.prompt_image = pygame.Surface((1, 1))
            self.p_image_rect = self.prompt_image.get_rect()
            self.reset_nesc = True

        self.locx = locx
        self.locy = locy


        self.message = ''
        self.shift = False
        self.letter_type = 's'
        self.have_message = False
        self.comp_message = ''

        self.font_size = 20
        self.font = pygame.font.Font(None, 20)
        self.color = [255, 0 , 0]
        self.blink_counter = 0
        self.blink_on = 1


        #all the one-time only needed drawing resources:
        self.prompt_surf = self.font.render(self.prompt,1,self.color)
        if self.reset_nesc == True:
            self.prompt_image = pygame.Surface((self.prompt_surf.get_width(), self.prompt_surf.get_height()))
            self.p_image_rect = self.prompt_surf.get_rect()
        self.p_image_rect[0] = self.locx
        self.p_image_rect[1] = self.locy
        self.cx = self.prompt_surf.get_width()/2
        self.b_image_rect[0] = self.locx + (self.cx * 2) + self.font_size
        self.b_image_rect[1] = self.locy
        self.input_surf = self.font.render(self.message,1,self.color)
        self.blinking_surf = self.font.render("|", 1, self.color)
        

        self.small_letters="abcdefghijklmnopqrstuvwxyz`1234567890-=[];',./\ " 
        self.big_letters='ABCDEFGHIJKLMNOPQRSTUVWXYZ~!@#$%^&*()_+{}:"<>?| ' 
        self.all_keys=[pygame.K_a, pygame.K_b, pygame.K_c, pygame.K_d,  
                      pygame.K_e, pygame.K_f, pygame.K_g, pygame.K_h, 
                  pygame.K_i,  
                      pygame.K_j, pygame.K_k, pygame.K_l, pygame.K_m, 
                  pygame.K_n,  
                      pygame.K_o, pygame.K_p, pygame.K_q, pygame.K_r, 
                  pygame.K_s,  
                      pygame.K_t, pygame.K_u, pygame.K_v, pygame.K_w, 
                  pygame.K_x,  
                      pygame.K_y, pygame.K_z, pygame.K_BACKQUOTE,pygame.K_1, 
                  pygame.K_2, 
                      pygame.K_3, pygame.K_4, pygame.K_5, pygame.K_6, 
                  pygame.K_7, 
                      pygame.K_8, pygame.K_9, pygame.K_0, pygame.K_MINUS, 
                  pygame.K_EQUALS, 
                      pygame.K_LEFTBRACKET, pygame.K_RIGHTBRACKET, 
                  pygame.K_SEMICOLON, 
                  pygame.K_QUOTE, pygame.K_COMMA, 
                      pygame.K_PERIOD, pygame.K_SLASH, pygame.K_BACKSLASH, 
                  pygame.K_SPACE]

    def Set_Font(self,font=None,color=None):
        if font is not None:
            self.font = font
            self.font_size = self.font.size("a")[1]
        if color is not None:
            self.color = color

        self.prompt_surf = self.font.render(self.prompt,1,self.color)
        if self.reset_nesc == True:
            self.prompt_image = pygame.Surface((self.prompt_surf.get_width(), self.prompt_surf.get_height()))
            self.p_image_rect = self.prompt_surf.get_rect()
        self.cx = self.prompt_surf.get_width()/2
        self.b_image_rect[0] = self.locx + (self.cx * 2) + self.font_size
        self.b_image_rect[1] = self.locy
        self.input_surf = self.font.render(self.message,1,self.color)
        self.blinking_surf = self.font.render("|", 1, self.color)

    def get_input(self):
        for event in pygame.event.get((KEYDOWN,KEYUP)): 
            if event.type==KEYDOWN: 
                if event.key==K_LSHIFT: 
                    self.letter_type = 'b'
                for e in range(len(self.all_keys)): 
                    if event.key==self.all_keys[e]:
                        if self.letter_type=="b":
                            if not len(self.message) == self.max_letters:
                                self.message = self.message+self.big_letters[e]
                        else:
                            if not len(self.message) == self.max_letters:
                                self.message = self.message+self.small_letters[e]
                             
                if event.key==K_BACKSPACE:
                    self.message=self.message[:-1]
                if event.key==K_RETURN:
                    self.comp_message = self.message
                    self.have_message = True
                    self.message=''
            elif event.type == KEYUP:
                if event.key == K_LSHIFT:
                    self.letter_type = 's'
            
        self.input_surf = self.font.render(self.message,1,self.color)

    def get_completed_message(self):
        if self.have_message == True:
            send_message = self.comp_message
            self.comp_message = ''
            self.have_message = False
            return send_message
        else:
            return None

    def draw(self, screen):
        self.blink_counter += 1
        if self.blink_counter == 25:
            self.blink_counter = 0
            if self.blink_on == 1:
                self.blink_on = 0
            else:
                self.blink_on = 1

        #draw prompt message and prompt image if not None
        screen.blit(self.prompt_image, self.p_image_rect)
        screen.blit(self.prompt_surf,(self.p_image_rect.centerx - self.cx, self.locy))
        #prompt stuff

        ##input stuff here
        screen.blit(self.box_image, self.b_image_rect)
        screen.blit(self.input_surf,(self.b_image_rect[0] + self.font_size, self.locy))
        ##input stuff

        #blinking '|'
        if self.blink_on == 1:
            screen.blit(self.blinking_surf,
                        (self.b_image_rect[0] + (self.font_size/1.3) + self.input_surf.get_width() + 5,
                         self.locy - (self.font_size/6.75)+4))
        
        return
    
